@@ -1,25 +1,27 @@
 # pylint: disable=no-member,unused-argument,too-many-lines,c-extension-no-member
 from __future__ import unicode_literals
-from typing import Dict, List
+
+from collections import OrderedDict
+
 import json
 import uuid
 from pathlib import Path
-from collections import OrderedDict
+from typing import Dict
+
 from django.contrib.auth import get_user_model
-from django.utils.functional import cached_property
 from django.db import transaction
-from rest_framework import serializers, exceptions, status, fields
-from vstutils.api import serializers as vst_serializers, fields as vst_fields
+from rest_framework import serializers, exceptions, fields
+
 from vstutils.api import auth as vst_auth
+from vstutils.api import serializers as vst_serializers, fields as vst_fields
 from vstutils.api.serializers import DataSerializer, EmptySerializer
-from vstutils.api.base import Response
-from ...main.utils import AnsibleArgumentsReference
-from ...main.settings import LANGUAGES
-from ...main.validators import path_validator
 
+from .base_serializers import with_signals, UserSerializer, _WithPermissionsSerializer, _SignalSerializer
 from ...main import models
-from ..signals import api_post_save, api_pre_save
-
+from ...main.settings import LANGUAGES
+from ...main.utils import AnsibleArgumentsReference
+from ...main.validators import path_validator
+from ...main.constants import HiddenArg, HiddenVar, CYPHER
 
 User = get_user_model()
 
@@ -59,17 +61,10 @@ class MultiTypeField(serializers.CharField):
         )
 
 
-class InventoryDependEnumField(vst_fields.DependEnumField):
-    def to_representation(self, value):
-        if isinstance(value, models.Inventory):
-            value = value.id  # nocv
-        return super(InventoryDependEnumField, self).to_representation(value)
-
-
-class InventoryAutoCompletionField(vst_fields.AutoCompletionField):
+class InventoryAutoCompletionField(vst_fields.VSTCharField):
 
     def to_internal_value(self, data):
-        inventory = super(InventoryAutoCompletionField, self).to_internal_value(data)
+        inventory = super().to_internal_value(data)
         try:
             inventory = models.Inventory.objects.get(id=int(inventory))
             user = self.context['request'].user
@@ -78,42 +73,22 @@ class InventoryAutoCompletionField(vst_fields.AutoCompletionField):
                     "You don't have permission to inventory."
                 )  # noce
         except (ValueError, KeyError):
-            self.check_path(inventory)
+            if ',' not in inventory:
+                path_validator(inventory)
         return inventory
 
-    def check_path(self, inventory):
-        if not hasattr(self.root, 'project'):  # nocv
-            return
-        self.root.project.check_path(inventory)
-
-
-def with_signals(func):
-    '''
-    Decorator for send api_pre_save and api_post_save signals from serializers.
-    '''
-    def func_wrapper(*args, **kwargs):
-        user = args[0].context['request'].user
-        with transaction.atomic():
-            instance = func(*args, **kwargs)
-            api_pre_save.send(
-                sender=instance.__class__, instance=instance, user=user
-            )
-        with transaction.atomic():
-            api_post_save.send(
-                sender=instance.__class__, instance=instance, user=user
-            )
-        return instance
-
-    return func_wrapper
-
 
 # Serializers
+class FactsSerializer(DataSerializer):
+    facts = serializers.JSONField(read_only=True)
+
+
 class ActionResponseSerializer(DataSerializer, EmptySerializer):
     detail = vst_fields.VSTCharField()
 
 
 class ExecuteResponseSerializer(ActionResponseSerializer):
-    history_id = vst_fields.RedirectIntegerField(default=None, allow_null=True)
+    history_id = vst_fields.RedirectIntegerField(default=None, allow_null=True, operation_name='history')
     executor = serializers.IntegerField(default=None, allow_null=True)
 
 
@@ -136,83 +111,37 @@ class SetOwnerSerializer(DataSerializer):
     def current_user(self) -> User:
         return self.context['request'].user
 
-    def to_representation(self, value: User):
+    def to_representation(self, value: User):  # pylint: disable=arguments-renamed
         return dict(user_id=value.pk)
 
     def to_internal_value(self, data: dict):
         return dict(pk=data['user_id'])
 
 
-class _SignalSerializer(serializers.ModelSerializer):
-    @cached_property
-    def _writable_fields(self) -> List:
-        writable_fields = super(_SignalSerializer, self)._writable_fields
-        fields_of_serializer = []
-        attrs = [
-            'field_name', 'source_attrs', 'source',
-            'read_only', 'required', 'write_only', 'default'
-        ]
-        for field in writable_fields:
-            if not isinstance(field, DataSerializer):
-                fields_of_serializer.append(field)
-                continue
-            field_object = serializers.DictField()
-            for attr in attrs:
-                setattr(field_object, attr, getattr(field, attr, None))
-            fields_of_serializer.append(field_object)
-        return fields_of_serializer
-
-    @with_signals
-    def create(self, validated_data):
-        return super(_SignalSerializer, self).create(validated_data)
-
-    @with_signals
-    def update(self, instance, validated_data):
-        return super(_SignalSerializer, self).update(instance, validated_data)
-
-
-class _WithPermissionsSerializer(_SignalSerializer):
-    perms_msg = "You do not have permission to perform this action."
-
-    def is_valid(self, *args, **kwargs):
-        result = super(_WithPermissionsSerializer, self).is_valid(*args, **kwargs)
-        if not hasattr(self, 'instance') or self.instance is None:  # noce
-            self.validated_data['owner'] = self.validated_data.get(
-                'owner', self.current_user()
-            )
-        return result
-
-    def current_user(self) -> User:
-        return self.context['request'].user  # noce
-
-
-class UserSerializer(vst_auth.UserSerializer):
-    is_staff = serializers.HiddenField(default=True, label='Staff')
-
-    @with_signals
-    def update(self, instance: User, validated_data: Dict):
-        return super(UserSerializer, self).update(instance, validated_data)
-
-
-class CreateUserSerializer(vst_auth.CreateUserSerializer):
+class CreateUserSerializer(vst_auth.CreateUserSerializer):  # noee
+    is_staff = None
 
     @with_signals
     def create(self, validated_data: Dict) -> User:
+        validated_data['is_staff'] = True
         return super().create(validated_data)
 
+    class Meta(vst_auth.CreateUserSerializer.Meta):
+        fields = tuple(filter(lambda field: field != 'is_staff', vst_auth.CreateUserSerializer.Meta.fields))
+
 
 class ChangePasswordSerializer(vst_auth.ChangePasswordSerializer):
 
     @with_signals
     def update(self, instance: User, validated_data: Dict) -> User:
-        return super(ChangePasswordSerializer, self).update(instance, validated_data)
+        return super().update(instance, validated_data)
 
 
 class OneUserSerializer(UserSerializer):
     email = serializers.EmailField(required=False)
 
     class Meta(vst_auth.OneUserSerializer.Meta):
-        pass
+        fields = tuple(filter(lambda field: field != 'is_staff', vst_auth.OneUserSerializer.Meta.fields))
 
 
 class ChartLineSettingSerializer(vst_serializers.JsonObjectSerializer):
@@ -249,15 +178,6 @@ class WidgetSettingsSerializer(vst_serializers.JsonObjectSerializer):
     pmwChartWidget = WidgetSettingSerializer()
 
 
-class UserSettingsSerializer(vst_serializers.JsonObjectSerializer):
-    lang = serializers.ChoiceField(choices=LANG_CHOICES, default=LANG_CHOICES[0])
-    autoupdateInterval = serializers.IntegerField(default=15000)
-    chartLineSettings = ChartLineSettingsSerializer()
-    widgetSettings = WidgetSettingsSerializer()
-    selectedSkin = serializers.CharField(required=False)
-    skinsSettings = vst_serializers.DataSerializer(required=False)
-
-
 class TeamSerializer(_WithPermissionsSerializer):
 
     class Meta:
@@ -284,6 +204,22 @@ class OneTeamSerializer(TeamSerializer):
 
 class HistorySerializer(_SignalSerializer):
     status = serializers.ChoiceField(choices=models.History.statuses, required=False)
+    executor = vst_fields.DependEnumField(field='initiator_type', types={
+        'project': vst_fields.FkModelField(select=UserSerializer,
+                                           autocomplete_property='id',
+                                           autocomplete_represent='username'),
+        'template': vst_fields.FkModelField(select=UserSerializer,
+                                            autocomplete_property='id',
+                                            autocomplete_represent='username'),
+        'scheduler': {
+            'type': 'string',
+            'x-format': 'static_value',
+            'x-options': {
+                'staticValue': 'system',
+                'realField': 'string',
+            }
+        },
+    })
 
     class Meta:
         model = models.History
@@ -321,8 +257,7 @@ class ProjectHistorySerializer(HistorySerializer):
         )
 
 
-class OneHistorySerializer(_SignalSerializer):
-    status = serializers.ChoiceField(choices=models.History.statuses, required=False)
+class OneHistorySerializer(HistorySerializer):
     raw_stdout = serializers.SerializerMethodField(read_only=True)
     execution_time = vst_fields.UptimeField()
 
@@ -347,15 +282,9 @@ class OneHistorySerializer(_SignalSerializer):
                   "raw_stdout",
                   "raw_inventory",)
 
-    def get_raw(self, request) -> str:
-        return self.instance.get_raw(request.query_params.get("color", "no") == "yes")
-
     def get_raw_stdout(self, obj: models.History):
         return self.context.get('request').build_absolute_uri("raw/")
 
-    def get_facts(self, request):
-        return self.instance.facts
-
 
 class HistoryLinesSerializer(_SignalSerializer):
     class Meta:
@@ -388,6 +317,7 @@ class HookSerializer(serializers.ModelSerializer):
 
 class VariableSerializer(_SignalSerializer):
     value = MultiTypeField(default="", allow_blank=True)
+    hidden_enum = HiddenVar
 
     class Meta:
         model = models.Variable
@@ -398,9 +328,9 @@ class VariableSerializer(_SignalSerializer):
         )
 
     def to_representation(self, instance: models.Variable):
-        result = super(VariableSerializer, self).to_representation(instance)
-        if instance.key in getattr(instance.content_object, 'HIDDEN_VARS', []):
-            result['value'] = "[~~ENCRYPTED~~]"
+        result = super().to_representation(instance)
+        if instance.key in self.hidden_enum.get_values():
+            result['value'] = CYPHER
         elif instance.key in getattr(instance.content_object, 'BOOLEAN_VARS', []):
             result['value'] = instance.value == 'True'
         return result
@@ -421,7 +351,7 @@ class InventoryVariableSerializer(VariableSerializer):
 
 
 class PeriodicTaskVariableSerializer(VariableSerializer):
-    pass
+    hidden_enum = HiddenArg
 
 
 class ProjectVariableSerializer(VariableSerializer):
@@ -435,15 +365,15 @@ class ProjectVariableSerializer(VariableSerializer):
     }, types={
         'repo_password': 'password',
         'repo_key': 'secretfile',
-        'repo_sync_on_run_timeout': 'uptime',
-        'ci_template': 'fk'
+        'repo_sync_on_run_timeout': vst_fields.UptimeField(default=0),
+        'ci_template': vst_fields.FkField(select='ExecutionTemplate')
     })
 
 
 class _WithVariablesSerializer(_WithPermissionsSerializer):
     @transaction.atomic
     def _do_with_vars(self, method_name: str, *args, **kwargs):
-        method = getattr(super(_WithVariablesSerializer, self), method_name)
+        method = getattr(super(), method_name)
         instance = method(*args, **kwargs)
         return instance
 
@@ -453,19 +383,13 @@ class _WithVariablesSerializer(_WithPermissionsSerializer):
     def update(self, instance, validated_data: Dict):
         return self._do_with_vars("update", instance, validated_data=validated_data)
 
-    def get_vars(self, representation):
-        return representation.get('vars', None)
+    def represent_vars(self, representation):
+        HiddenVar.hide_values(representation.get('vars'))
 
-    def to_representation(self, instance, hidden_vars: List[str] = None):
-        rep = super(_WithVariablesSerializer, self).to_representation(instance)
-        hv = hidden_vars
-        hv = getattr(instance, 'HIDDEN_VARS', []) if hidden_vars is None else hidden_vars
-        vars = self.get_vars(rep)
-        if vars is not None:
-            for mask_key in hv:
-                if mask_key in vars.keys():
-                    vars[mask_key] = "[~~ENCRYPTED~~]"
-        return rep
+    def to_representation(self, instance):
+        representation = super().to_representation(instance)
+        self.represent_vars(representation)
+        return representation
 
 
 class HostSerializer(_WithVariablesSerializer):
@@ -474,7 +398,7 @@ class HostSerializer(_WithVariablesSerializer):
         required=False,
         default='HOST'
     )
-    from_project = serializers.BooleanField(read_only=True, label='Project Based')
+    from_project = serializers.BooleanField(read_only=True, label='Project based')
 
     class Meta:
         model = models.Host
@@ -525,120 +449,7 @@ class ModuleSerializer(vst_serializers.VSTSerializer):
         )
 
 
-class OneModuleSerializer(ModuleSerializer):
-    data = fields.JSONField(read_only=True)
-
-    class Meta:
-        model = models.Module
-        fields = (
-            'id',
-            'path',
-            'name',
-            'data',
-        )
-
-
-class PeriodictaskSerializer(_WithVariablesSerializer):
-    kind = serializers.ChoiceField(
-        choices=[(k, k) for k in models.PeriodicTask.kinds],
-        required=False,
-        default=models.PeriodicTask.kinds[0],
-        label='Task type'
-    )
-    type = serializers.ChoiceField(
-        choices=[(k, k) for k in models.PeriodicTask.types],
-        required=False,
-        default=models.PeriodicTask.types[0],
-        label='Interval type'
-    )
-
-    template_opt = vst_fields.DependEnumField(
-        allow_blank=True,
-        required=False,
-        allow_null=True,
-        field='kind',
-        types={
-            'PLAYBOOK': 'hidden',
-            'MODULE': 'hidden',
-            'TEMPLATE': 'autocomplete',
-        }
-    )
-
-    schedule = vst_fields.DependEnumField(
-        allow_blank=True, field='type', types={
-            'CRONTAB': 'crontab',
-            'INTERVAL': 'integer',
-        }
-    )
-
-    mode = vst_fields.DependEnumField(
-        allow_blank=True, required=False, field='kind', types={
-            'PLAYBOOK': 'fk_autocomplete',
-            'MODULE': 'fk_autocomplete',
-            'TEMPLATE': 'hidden',
-        }
-    )
-
-    inventory = InventoryDependEnumField(
-        allow_blank=True, required=False, field='kind', types={
-            'PLAYBOOK': 'fk_autocomplete',
-            'MODULE': 'fk_autocomplete',
-            'TEMPLATE': 'hidden',
-        }
-    )
-
-    class Meta:
-        model = models.PeriodicTask
-        fields = ('id',
-                  'name',
-                  'kind',
-                  'mode',
-                  'inventory',
-                  'save_result',
-                  'template',
-                  'template_opt',
-                  'enabled',
-                  'type',
-                  'schedule',)
-
-    @transaction.atomic
-    def _do_with_vars(self, *args, **kwargs):
-        kw = kwargs['validated_data']
-        if kw.get('kind', None) == 'TEMPLATE':
-            kw['inventory'] = ''
-            kw['mode'] = ''
-            kwargs['validated_data'] = kw
-        return super(PeriodictaskSerializer, self)._do_with_vars(*args, **kwargs)
-
-
-class OnePeriodictaskSerializer(PeriodictaskSerializer):
-    notes = vst_fields.TextareaField(required=False, allow_blank=True)
-
-    class Meta:
-        model = models.PeriodicTask
-        fields = ('id',
-                  'name',
-                  'kind',
-                  'mode',
-                  'inventory',
-                  'save_result',
-                  'template',
-                  'template_opt',
-                  'enabled',
-                  'type',
-                  'schedule',
-                  'notes',)
-
-    def execute(self) -> Response:
-        inventory = self.instance.inventory
-        rdata = ExecuteResponseSerializer(data=dict(
-            detail="Started at inventory {}.".format(inventory),
-            history_id=self.instance.execute(sync=False)
-        ))
-        rdata.is_valid(True)
-        return Response(rdata.data, status.HTTP_201_CREATED)
-
-
+# NOTE: deprecated. Remove this with tests after breaking v2 api support
 class TemplateSerializer(_WithVariablesSerializer):
     data = DataSerializer(required=True, write_only=True)
     options = DataSerializer(write_only=True)
@@ -661,35 +472,16 @@ class TemplateSerializer(_WithVariablesSerializer):
             'options_list',
         )
 
-    def get_vars(self, representation):
-        try:
-            return representation['data']['vars']
-        except KeyError:  # nocv
-            return None
+    def represent_vars(self, representation):
+        if 'data' in representation:
+            HiddenArg.hide_values(representation['data'].get('vars'))
 
-    def set_opts_vars(self, rep, hidden_vars: List[str]):
-        if not rep.get('vars', None):
-            return rep
-        var = rep['vars']
-        for mask_key in hidden_vars:
-            if mask_key in var.keys():
-                var[mask_key] = "[~~ENCRYPTED~~]"
-        return rep
-
-    def repr_options(self, instance: models.Template, data: Dict, hidden_vars: List):
-        hv = hidden_vars
-        hv = instance.HIDDEN_VARS if hv is None else hv
-        for name, rep in data.get('options', {}).items():
-            data['options'][name] = self.set_opts_vars(rep, hv)
-
-    def to_representation(self, instance, hidden_vars: List[str] = None) -> OrderedDict:
+    def to_representation(self, instance) -> OrderedDict:
         data = OrderedDict()
         if instance.kind in ["Task", "Module"]:
-            hidden_vars = models.PeriodicTask.HIDDEN_VARS
-            data = super(TemplateSerializer, self).to_representation(
-                instance, hidden_vars=hidden_vars
-            )
-            self.repr_options(instance, data, hidden_vars)
+            data = super().to_representation(instance)
+            for option in data.get('options', {}).values():
+                HiddenArg.hide_values(option.get('vars'))
         return data
 
 
@@ -711,15 +503,13 @@ class OneTemplateSerializer(TemplateSerializer):
             'options_list',
         )
 
-    def execute(self, request):
-        return self.instance.execute(request.user, request.data.get('option', None))
-
 
 class TemplateExecSerializer(DataSerializer):
-    option = vst_fields.VSTCharField(
-        help_text='Option name from template options.',
-        min_length=0, allow_blank=True,
-        required=False
+    option = vst_fields.FkField(
+        select='TemplateOption',
+        autocomplete_property='name',
+        autocomplete_represent='name',
+        allow_null=True,
     )
 
 
@@ -730,44 +520,8 @@ class _InventoryOperations(_WithVariablesSerializer):
     pass
 
 
-###################################
-
-class GroupSerializer(_WithVariablesSerializer):
-    children = serializers.BooleanField(read_only=True)
-    from_project = serializers.BooleanField(read_only=True, label='Project Based')
-
-    class Meta:
-        model = models.Group
-        fields = ('id',
-                  'name',
-                  'children',
-                  'from_project')
-
-
-class OneGroupSerializer(GroupSerializer, _InventoryOperations):
-    owner = UserSerializer(read_only=True)
-    notes = vst_fields.TextareaField(required=False, allow_blank=True)
-
-    class Meta:
-        model = models.Group
-        fields = ('id',
-                  'name',
-                  'notes',
-                  'children',
-                  'owner',)
-
-    class ValidationException(exceptions.ValidationError):
-        status_code = 409
-
-
-class GroupCreateMasterSerializer(OneGroupSerializer):
-    children = serializers.BooleanField(write_only=True,
-                                        label='Contains groups',
-                                        default=False)
-
-
 class InventorySerializer(_WithVariablesSerializer):
-    from_project = serializers.BooleanField(read_only=True, label='Project Based')
+    from_project = serializers.BooleanField(read_only=True, label='Project based')
 
     class Meta:
         model = models.Inventory
@@ -788,42 +542,75 @@ class OneInventorySerializer(InventorySerializer, _InventoryOperations):
                   'owner',)
 
 
-class ProjectCreateMasterSerializer(vst_serializers.VSTSerializer, _WithPermissionsSerializer):
+class ProjectCreateMasterSerializer(_WithPermissionsSerializer):
     types = models.list_to_choices(models.Project.repo_handlers.keys())
-    auth_types = ['NONE', 'KEY', 'PASSWORD']
-    branch_auth_types = {t: "hidden" for t in models.Project.repo_handlers.keys()}
-    branch_auth_types['GIT'] = 'string'
-    branch_types = dict(**branch_auth_types)
-    branch_types['TAR'] = 'string'
 
     status = vst_fields.VSTCharField(read_only=True)
     type = serializers.ChoiceField(choices=types, default='MANUAL', label='Repo type')
-    repository = vst_fields.VSTCharField(default='MANUAL', label='Repo url')
-    repo_auth = vst_fields.DependEnumField(default='NONE',
-                                           field='type',
-                                           choices={"GIT": auth_types},
-                                           types=branch_auth_types,
-                                           label='Repo auth type',
-                                           write_only=True)
-    auth_data = vst_fields.DependEnumField(allow_blank=True,
-                                           write_only=True,
-                                           default='',
-                                           field='repo_auth',
-                                           label='Repo auth data',
-                                           types={
-                                               'KEY': 'secretfile',
-                                               'PASSWORD': 'password',
-                                               'NONE': 'hidden'
-                                           })
-    branch = vst_fields.DependEnumField(allow_blank=True,
-                                        required=False,
-                                        allow_null=True,
-                                        label='Branch for GIT(branch/tag/SHA) or TAR(subdir)',
-                                        field='type',
-                                        types=branch_types)
-    additional_playbook_path = vst_fields.VSTCharField(required=False,
-                                                       allow_null=True,
-                                                       label='Directory with playbooks')
+    repository = vst_fields.DependEnumField(
+        field='type',
+        default='MANUAL',
+        label='Repo url',
+        types={'MANUAL': 'hidden', 'GIT': 'string', 'TAR': 'string'},
+    )
+    repo_auth = vst_fields.DependEnumField(
+        field='type',
+        default='NONE',
+        types={
+            'MANUAL': {'type': 'string', 'format': 'hidden'},
+            'TAR': {'type': 'string', 'format': 'hidden'},
+            'GIT': {
+                'type': 'string',
+                'enum': ('NONE', 'KEY', 'PASSWORD'),
+                'default': 'NONE',
+            },
+        },
+        label='Repo auth type',
+        write_only=True
+    )
+    auth_data = vst_fields.DependEnumField(
+        allow_blank=True,
+        write_only=True,
+        default='',
+        field='repo_auth',
+        label='Repo auth data',
+        types={
+            'KEY': 'secretfile',
+            'PASSWORD': 'password',
+            'NONE': 'hidden'
+        }
+    )
+    branch = vst_fields.DependEnumField(
+        allow_blank=True,
+        required=False,
+        allow_null=True,
+        label='Branch for GIT (branch/tag/SHA) or TAR (subdir)',
+        field='type',
+        types={
+            'MANUAL': {'type': 'string', 'format': 'hidden'},
+            'GIT': {'type': 'string'},
+            'TAR': {'type': 'string'},
+        }
+    )
+    additional_playbook_path = vst_fields.VSTCharField(
+        required=False,
+        allow_null=True,
+        label='Directory with playbooks'
+    )
+
+    _schema_properties_groups = {
+        'General': (
+            'name',
+            'type',
+            'additional_playbook_path',
+        ),
+        'Repository': (
+            'repository',
+            'branch',
+            'repo_auth',
+            'auth_data',
+        ),
+    }
 
     class Meta:
         model = models.Project
@@ -839,7 +626,7 @@ class ProjectCreateMasterSerializer(vst_serializers.VSTSerializer, _WithPermissi
             'additional_playbook_path',
         )
         extra_kwargs = {
-            'name': {'required': True}
+            'name': {'required': True},
         }
 
     @transaction.atomic
@@ -850,7 +637,7 @@ class ProjectCreateMasterSerializer(vst_serializers.VSTSerializer, _WithPermissi
         repo_branch = validated_data.pop('branch', None)
         playbook_path = validated_data.pop('additional_playbook_path', '')
 
-        instance = super(ProjectCreateMasterSerializer, self).create(validated_data)
+        instance = super().create(validated_data)
         instance.variables.create(key='repo_type', value=repo_type)
         if repo_auth_type != 'NONE':  # nocv
             key = 'repo_{}'.format(repo_auth_type.lower())
@@ -935,7 +722,7 @@ class ProjectSerializer(_InventoryOperations):
 
     @transaction.atomic
     def _do_with_vars(self, *args, **kw):
-        instance = super(ProjectSerializer, self)._do_with_vars(*args, **kw)
+        instance = super()._do_with_vars(*args, **kw)
         return instance if instance.repo_class else None
 
 
@@ -959,67 +746,8 @@ class OneProjectSerializer(ProjectSerializer, _InventoryOperations):
                   'readme_content',
                   'execute_view_data',)
 
-    @transaction.atomic()
-    def sync(self) -> Response:
-        self.instance.start_repo_task("sync")
-        serializer = ActionResponseSerializer(
-            data=dict(detail="Sync with {}.".format(self.instance.repository))
-        )
-        serializer.is_valid(True)
-        return Response(serializer.data, status.HTTP_200_OK)
 
-    def _get_ansible_serializer(self, kind: str) -> serializers.Serializer:
-        view = self.context['view']
-        exec_method = getattr(view, 'execute_{}'.format(kind), None)
-        if exec_method is None:  # nocv
-            raise Exception('Unknown kind')
-        serializer_class = exec_method.kwargs['serializer_class']
-        serializer = serializer_class(context=self.context)
-        serializer.project = self.instance
-        return serializer
-
-    def _execution(self, kind: str, data: Dict, user: User, **kwargs) -> Response:
-        template = data.pop("template", None)
-        inventory = data.get("inventory", None)
-        msg = "Started in the inventory {}.".format(
-            inventory if inventory else 'specified in the project configuration.'
-        )
-        if template is not None:
-            init_type = "template"
-            obj_id = template
-            msg = 'Start template [id={}].'.format(template)
-        else:
-            init_type = "project"
-            obj_id = self.instance.id
-            serializer = self._get_ansible_serializer(kind.lower())
-            data = {
-                k: v for k, v in serializer.to_internal_value(data).items()
-                if k in data.keys() or v
-            }
-        target = data.pop(kind)
-        try:
-            target = str(target)
-        except UnicodeEncodeError:  # nocv
-            target = target.encode('utf-8')
-        history_id = self.instance.execute(
-            kind, str(target),
-            initiator=obj_id, initiator_type=init_type, executor=user, **data
-        )
-        rdata = ExecuteResponseSerializer(data=dict(
-            detail=msg,
-            history_id=history_id, executor=user.id
-        ))
-        rdata.is_valid(raise_exception=True)
-        return Response(rdata.data, status.HTTP_201_CREATED)
-
-    def execute_playbook(self, request) -> Response:
-        return self._execution("playbook", dict(request.data), request.user)
-
-    def execute_module(self, request) -> Response:
-        return self._execution("module", dict(request.data), request.user)
-
-
-def generate_fileds(ansible_reference: AnsibleArgumentsReference, ansible_type: str) -> OrderedDict:
+def generate_fileds(ansible_reference: AnsibleArgumentsReference, ansible_type: str, no_default=False) -> OrderedDict:
     if ansible_type is None:
         return OrderedDict()  # nocv
 
@@ -1036,17 +764,16 @@ def generate_fileds(ansible_reference: AnsibleArgumentsReference, ansible_type:
             kwargs['default'] = False
         elif ref_type == 'int':
             field = serializers.IntegerField
-        elif ref_type == 'string' or 'choice':
+        elif ref_type in ('string', 'choice'):
             field = vst_fields.VSTCharField
             kwargs['allow_blank'] = True
 
         if ref == 'verbose':
             field = serializers.IntegerField
             kwargs.update(dict(max_value=4, default=0))
-        if ref in models.PeriodicTask.HIDDEN_VARS:
+        if ref in HiddenArg.get_values():
             field = vst_fields.SecretFileInString
         if ref == 'inventory':
-            kwargs['autocomplete'] = 'Inventory'
             field = InventoryAutoCompletionField
 
         if field is None:  # nocv
@@ -1057,6 +784,8 @@ def generate_fileds(ansible_reference: AnsibleArgumentsReference, ansible_type:
                 kwargs['default'] = 'all'
 
         field_name = ref.replace('-', '_')
+        if no_default:
+            kwargs['default'] = fields.empty
         fields_of_serializer[field_name] = field(**kwargs)
 
     return fields_of_serializer
@@ -1135,7 +864,7 @@ class InventoryImportSerializer(serializers.Serializer):
     # pylint: disable=abstract-method
     inventory_id = vst_fields.RedirectIntegerField(default=None, allow_null=True, read_only=True)
     name = serializers.CharField(required=True)
-    raw_data = vst_fields.VSTCharField()
+    raw_data = vst_fields.FileInStringField()
 
     def create(self, validated_data: Dict) -> Dict:
         return models.Inventory.import_inventory_from_string(**validated_data)
