@@ -1,37 +1,39 @@
 # pylint: disable=unused-argument,protected-access,too-many-ancestors
 from collections import OrderedDict
-from django.http import HttpResponse
+
+from django.db import transaction
+from django.http.response import HttpResponse
 from django.utils.decorators import method_decorator
-from rest_framework import exceptions as excepts, status, permissions
+from rest_framework import exceptions as excepts, status
 from rest_framework.authtoken import views as token_views
 from drf_yasg.utils import swagger_auto_schema
 from vstutils.api import auth as vst_auth
 from vstutils.api.permissions import StaffPermission
-from vstutils.api import base, views, serializers as vstsers, decorators as deco, responses
+from vstutils.api import base, serializers as vstsers, decorators as deco, responses
 from vstutils.utils import KVExchanger
+from vstutils.api.responses import HTTP_200_OK, HTTP_201_CREATED, HTTP_204_NO_CONTENT
 
 from . import filters
-from .permissions import InventoryItemsPermission
+from .permissions import InventoryItemsPermission, CreateUsersPermission
 from . import serializers as sers
+from ..v3 import serializers as sers3
 from ...main import utils
 
 yes = True
 no = False
-default_action = dict(methods=["post"], detail=yes)
-action_kw = dict(**default_action)
-action_kw.update(dict(
-    response_serializer=sers.ActionResponseSerializer,
-    response_code=status.HTTP_200_OK
-))
-execute_kw = dict(**default_action)
-execute_kw.update(dict(
-    response_serializer=sers.ExecuteResponseSerializer,
-    response_code=status.HTTP_201_CREATED
-))
+default_action = {'methods': ["post"], 'detail': yes}
+action_kw = {**default_action}
+action_kw.update({'response_serializer': sers.ActionResponseSerializer, 'response_code': status.HTTP_200_OK})
+execute_kw = default_action.copy()
+execute_kw.update({
+    'serializer_class': sers.EmptySerializer,
+    'response_serializer': sers.ExecuteResponseSerializer,
+    'response_code': status.HTTP_201_CREATED
+})
 
 
 def concat_classes(*args):
-    concated_object = list()
+    concated_object = []
     for arg in args:
         if isinstance(arg, (list, tuple)):
             concated_object += arg
@@ -42,7 +44,7 @@ def concat_classes(*args):
 
 class _VariablesCopyMixin(base.CopyMixin):
     def copy_instance(self, instance):
-        new_instance = super(_VariablesCopyMixin, self).copy_instance(instance)
+        new_instance = super().copy_instance(instance)
         new_instance.variables.bulk_create([
             sers.models.Variable(key=key, value=value, content_object=new_instance)
             for key, value in instance.vars.items()
@@ -56,19 +58,19 @@ class OwnedView(base.ModelViewSet, base.CopyMixin):
     @deco.action(methods=["post"], detail=True, serializer_class=sers.SetOwnerSerializer)
     def set_owner(self, request, **kwargs):
         # pylint: disable=unused-argument
-        '''
+        """
         Change instance owner.
-        '''
+        """
         serializer = sers.SetOwnerSerializer(
             self.get_object(), data=request.data, context=self.get_serializer_context()
         )
         serializer.is_valid(True)
         serializer.save()
-        return base.Response(serializer.data, status.HTTP_201_CREATED).resp
+        return HTTP_201_CREATED(serializer.data)
 
 
 class __VarsViewSet(base.ModelViewSet):
-    '''
+    """
     Instance execution variables.
 
     list:
@@ -88,14 +90,15 @@ class __VarsViewSet(base.ModelViewSet):
 
     destroy:
         Remove an existing variable.
-    '''
+    """
     model = sers.models.Variable
     serializer_class = sers.VariableSerializer
-    filter_class = filters.VariableFilter
+    filterset_class = filters.VariableFilter
+    optimize_get_by_values = False
 
 
 class __InvVarsViewSet(__VarsViewSet):
-    '''
+    """
     Inventory hosts variables.
 
     retrieve:
@@ -115,12 +118,12 @@ class __InvVarsViewSet(__VarsViewSet):
 
     update:
         Update variable value.
-    '''
+    """
     serializer_class = sers.InventoryVariableSerializer
 
 
 class __PeriodicTaskVarsViewSet(__VarsViewSet):
-    '''
+    """
     Periodic task additional execute variables.
 
     retrieve:
@@ -140,12 +143,12 @@ class __PeriodicTaskVarsViewSet(__VarsViewSet):
 
     update:
         Update variable value.
-    '''
+    """
     serializer_class = sers.PeriodicTaskVariableSerializer
 
 
 class __ProjectVarsViewSet(__VarsViewSet):
-    '''
+    """
     Project settings variables.
 
     retrieve:
@@ -165,7 +168,7 @@ class __ProjectVarsViewSet(__VarsViewSet):
 
     update:
         Update variable value.
-    '''
+    """
     serializer_class = sers.ProjectVariableSerializer
 
 
@@ -177,14 +180,12 @@ class TokenView(token_views.ObtainAuthToken):
         if token:
             key = token.key
             token.delete()
-            return base.Response(
-                "Token {} removed.".format(key), status.HTTP_204_NO_CONTENT
-            ).resp
+            return HTTP_204_NO_CONTENT(f"Token {key} removed.")
         raise excepts.ParseError("Token not found.")
 
 
 class UserViewSet(vst_auth.UserViewSet, base.CopyMixin):
-    '''
+    """
     retrieve:
         Return a user instance.
 
@@ -202,41 +203,20 @@ class UserViewSet(vst_auth.UserViewSet, base.CopyMixin):
 
     update:
         Update a user.
-    '''
+    """
     serializer_class = sers.UserSerializer
     serializer_class_one = sers.OneUserSerializer
     serializer_class_create = sers.CreateUserSerializer
     serializer_class_change_password = sers.ChangePasswordSerializer  # pylint: disable=invalid-name
+    permission_classes = vst_auth.UserViewSet.permission_classes + (CreateUsersPermission, )
 
     copy_related = ['groups']
     copy_field_name = 'username'
 
-    def copy_instance(self, instance):
-        new_instance = super(UserViewSet, self).copy_instance(instance)
-        new_instance.settings.data = instance.settings.get_settings_copy()
-        new_instance.settings.save()
-        return new_instance
-
-    @deco.action(
-        ["post", "delete", "get"], url_path="settings",
-        detail=yes, serializer_class=sers.UserSettingsSerializer,
-        permission_classes=(permissions.IsAuthenticated,)
-    )
-    def user_settings(self, request, *args, **kwargs):
-        '''
-        Return user settings.
-        '''
-        obj = self.get_object()
-        method = request.method
-        if method != "GET":
-            obj.settings.data = request.data if method == "POST" else {}
-            obj.settings.save()
-        return base.Response(obj.settings.data, status.HTTP_200_OK).resp
-
 
 @deco.nested_view('user', 'id', allow_append=yes, manager_name='users', view=UserViewSet)
 class TeamViewSet(OwnedView):
-    '''
+    """
     retrieve:
         Return a team instance.
 
@@ -254,11 +234,11 @@ class TeamViewSet(OwnedView):
 
     update:
         Update a team.
-    '''
+    """
     model = sers.models.UserGroup
     serializer_class = sers.TeamSerializer
     serializer_class_one = sers.OneTeamSerializer
-    filter_class = filters.TeamFilter
+    filterset_class = filters.TeamFilter
     copy_related = ['users']
 
 
@@ -266,14 +246,14 @@ class __HistoryLineViewSet(base.ReadOnlyModelViewSet):
     schema = None
     model = sers.models.HistoryLines
     serializer_class = sers.HistoryLinesSerializer
-    filter_class = filters.HistoryLinesFilter
+    filterset_class = filters.HistoryLinesFilter
 
 
 @method_decorator(name='lines_list', decorator=swagger_auto_schema(auto_schema=None))
 @method_decorator(name='raw', decorator=swagger_auto_schema(auto_schema=None))
 @deco.nested_view('lines', manager_name='raw_history_line', view=__HistoryLineViewSet)
 class HistoryViewSet(base.HistoryModelViewSet):
-    '''
+    """
 
     retrieve:
         Return a execution history instance.
@@ -284,44 +264,43 @@ class HistoryViewSet(base.HistoryModelViewSet):
     destroy:
         Remove an existing history record.
 
-    '''
+    """
     model = sers.models.History
     serializer_class = sers.HistorySerializer
     serializer_class_one = sers.OneHistorySerializer
-    filter_class = filters.HistoryFilter
+    filterset_class = filters.HistoryFilter
     POST_WHITE_LIST = ['cancel']
 
-    @deco.action(["get"], detail=yes, serializer_class=sers.EmptySerializer)
+    @deco.action(detail=yes, serializer_class=sers.EmptySerializer)
     def raw(self, request, *args, **kwargs):
-        '''
+        """
         RAW executions output.
-        '''
-        result = self.get_serializer(self.get_object()).get_raw(request)
-        return HttpResponse(result, content_type="text/plain")
+        """
+        return HttpResponse(self.get_raw(request), content_type="text/plain", status=200)
 
     @deco.subaction(serializer_class=sers.EmptySerializer, **action_kw)
     def cancel(self, request, *args, **kwargs):
-        '''
+        """
         Cencel working task.
-        '''
+        """
         obj = self.get_object()
         exch = KVExchanger(utils.CmdExecutor.CANCEL_PREFIX + str(obj.id))
         exch.send(True, 60) if obj.working else None
-        return base.Response("Task canceled: {}".format(obj.id), status.HTTP_200_OK).resp
+        return HTTP_200_OK(f"Task canceled: {obj.id}")
 
-    @deco.action(["get"], detail=yes, serializer_class=sers.DataSerializer)
+    @deco.action(["get", "head"], detail=yes, serializer_class=sers.FactsSerializer)
     def facts(self, request, *args, **kwargs):
-        '''
+        """
         Get compilated history facts (only for execution 'module' with module 'setup').
-        '''
-        objs = self.get_serializer(self.get_object()).get_facts(request)
-        return base.Response(objs, status.HTTP_200_OK).resp
+        """
+        serializer = self.get_serializer(instance=self.get_object().facts)
+        return HTTP_200_OK({'facts': serializer.data})
 
-    @deco.subaction(["delete"], detail=yes, serializer_class=sers.EmptySerializer)
+    @deco.subaction(methods=["delete"], detail=yes, serializer_class=sers.EmptySerializer)
     def clear(self, request, *args, **kwargs):
-        '''
+        """
         Clear history output.
-        '''
+        """
         default_message = "Output trancated.\n"
         obj = self.get_object()
         if obj.status in ["RUN", "DELAY"] or obj.raw_stdout == default_message:  # nocv
@@ -329,13 +308,15 @@ class HistoryViewSet(base.HistoryModelViewSet):
                 "Job is running or already trancated"
             )
         obj.raw_stdout = default_message
-        result = self.get_serializer(obj).get_raw(request)
-        return base.Response(result, status.HTTP_204_NO_CONTENT).resp
+        return HTTP_204_NO_CONTENT(self.get_raw(request))
+
+    def get_raw(self, request):
+        return self.get_object().get_raw(request.query_params.get("color", "no") == "yes")
 
 
 @deco.nested_view('variables', 'id', view=__InvVarsViewSet)
 class HostViewSet(OwnedView, _VariablesCopyMixin):
-    '''
+    """
     retrieve:
         Return a host instance.
 
@@ -353,11 +334,11 @@ class HostViewSet(OwnedView, _VariablesCopyMixin):
 
     update:
         Update a host.
-    '''
+    """
     model = sers.models.Host
     serializer_class = sers.HostSerializer
     serializer_class_one = sers.OneHostSerializer
-    filter_class = filters.HostFilter
+    filterset_class = filters.HostFilter
     permission_classes = concat_classes(
         OwnedView.permission_classes,
         _VariablesCopyMixin.permission_classes,
@@ -366,8 +347,8 @@ class HostViewSet(OwnedView, _VariablesCopyMixin):
 
 
 @deco.nested_view('variables', 'id', view=__InvVarsViewSet)
-class _BaseGroupViewSet(OwnedView, base.ModelViewSet):
-    '''
+class _BaseGroupViewSet(OwnedView, sers.models.Group.generated_view):  # pylint: disable=inherit-non-class
+    """
     retrieve:
         Return a group instance.
 
@@ -385,23 +366,30 @@ class _BaseGroupViewSet(OwnedView, base.ModelViewSet):
 
     update:
         Update a group.
-    '''
-    model = sers.models.Group
-    serializer_class = sers.GroupSerializer
-    serializer_class_one = sers.OneGroupSerializer
-    serializer_class_create = sers.GroupCreateMasterSerializer
-    filter_class = filters.GroupFilter
+    """
+    filterset_class = filters.GroupFilter
+
+    class ValidationException(excepts.ValidationError):
+        status_code = 409
 
 
-@deco.nested_view('host', 'id', manager_name='hosts', allow_append=yes, view=HostViewSet)
-@deco.nested_view('group', 'id', manager_name='groups', allow_append=yes, view=_BaseGroupViewSet)
+@deco.nested_view('hosts', 'id', allow_append=yes, view=HostViewSet)
 class _GroupMixin(OwnedView, _VariablesCopyMixin):
-    '''
+    """
     Instance with groups and hosts.
-    '''
+    """
     copy_related = ['hosts', 'groups']
 
 
+def nested_allow_check(view):
+    # pylint: disable=no-member
+    exception = _BaseGroupViewSet.ValidationException
+    if not view.nested_parent_object.children and view.nested_name == 'groups':
+        raise exception("Group is not children.")
+    if view.nested_parent_object.children and view.nested_name == 'hosts':
+        raise exception("Group is children.")
+
+
 class GroupViewSet(_BaseGroupViewSet, _GroupMixin):
     __doc__ = _BaseGroupViewSet.__doc__
     permission_classes = concat_classes(
@@ -410,20 +398,15 @@ class GroupViewSet(_BaseGroupViewSet, _GroupMixin):
         InventoryItemsPermission
     )
 
-    def nested_allow_check(self):
-        # pylint: disable=no-member
-        exception = _BaseGroupViewSet.serializer_class_one.ValidationException
-        if not self.nested_parent_object.children and self.nested_name == 'group':
-            raise exception("Group is not children.")
-        if self.nested_parent_object.children and self.nested_name == 'host':
-            raise exception("Group is children.")
+    nested_allow_check = nested_allow_check
 
 
 @deco.nested_view('all_groups', 'id', methods=['get'], view=GroupViewSet, subs=None)
 @deco.nested_view('all_hosts', 'id', methods=['get'], view=HostViewSet, subs=None)
+@deco.nested_view('group', 'id', manager_name='groups', allow_append=True, view=GroupViewSet)
 @deco.nested_view('variables', 'id', view=__InvVarsViewSet)
 class InventoryViewSet(_GroupMixin):
-    '''
+    """
     retrieve:
         Return a inventory instance.
 
@@ -441,13 +424,12 @@ class InventoryViewSet(_GroupMixin):
 
     update:
         Update a inventory.
-    '''
+    """
     model = sers.models.Inventory
     serializer_class = sers.InventorySerializer
     serializer_class_one = sers.OneInventorySerializer
     serializer_class_import_inventory = sers.InventoryImportSerializer  # pylint: disable=invalid-name
-    filter_class = filters.InventoryFilter
-    copy_related = ['hosts', 'groups']
+    filterset_class = filters.InventoryFilter
     permission_classes = concat_classes(
         _GroupMixin.permission_classes,
         InventoryItemsPermission
@@ -485,7 +467,7 @@ class __ProjectInventoryViewSet(InventoryViewSet):
 
 
 class __PlaybookViewSet(base.ReadOnlyModelViewSet):
-    '''
+    """
     Ansible playbook for project.
 
     retrieve:
@@ -493,16 +475,16 @@ class __PlaybookViewSet(base.ReadOnlyModelViewSet):
 
     list:
         Return all playbooks of project.
-    '''
+    """
     lookup_field = 'id'
     model = sers.models.Task
     serializer_class = sers.PlaybookSerializer
     serializer_class_one = sers.OnePlaybookSerializer
-    filter_class = filters.TaskFilter
+    filterset_class = filters.TaskFilter
 
 
 class __ModuleViewSet(base.ReadOnlyModelViewSet):
-    '''
+    """
     Ansible module for project.
 
     retrieve:
@@ -510,17 +492,17 @@ class __ModuleViewSet(base.ReadOnlyModelViewSet):
 
     list:
         Return all available modules of project.
-    '''
+    """
     lookup_field = 'id'
     model = sers.models.Module
     serializer_class = sers.ModuleSerializer
-    serializer_class_one = sers.OneModuleSerializer
-    filter_class = filters.ModuleFilter
+    serializer_class_one = sers3.OneModuleSerializer
+    filterset_class = filters.ModuleFilter
 
 
 @deco.nested_view('variables', 'id', view=__PeriodicTaskVarsViewSet)
 class __PeriodicTaskViewSet(base.ModelViewSet):
-    '''
+    """
     retrieve:
         Return a perodic task instance.
 
@@ -539,24 +521,27 @@ class __PeriodicTaskViewSet(base.ModelViewSet):
     update:
         Update a periodic task.
 
-    '''
+    """
     lookup_field = 'id'
     model = sers.models.PeriodicTask
-    serializer_class = sers.PeriodictaskSerializer
-    serializer_class_one = sers.OnePeriodictaskSerializer
-    filter_class = filters.PeriodicTaskFilter
+    serializer_class = sers3.PeriodictaskSerializer
+    serializer_class_one = sers3.OnePeriodictaskSerializer
+    filterset_class = filters.PeriodicTaskFilter
 
-    @deco.subaction(serializer_class=sers.EmptySerializer, **execute_kw)
+    @deco.subaction(**{
+        **execute_kw,
+        'response_serializer': sers.ExecuteResponseSerializer,
+        'serializer_class': sers.EmptySerializer
+    })
     def execute(self, request, *args, **kwargs):
-        '''
+        """
         Ad-hoc execute periodic task.
-        '''
-        serializer = self.get_serializer(self.get_object())
-        return serializer.execute().resp
+        """
+        return HTTP_201_CREATED(sers3.OnePeriodictaskSerializer(self.get_object()).execute())
 
 
 class __TemplateViewSet(base.ModelViewSet):
-    '''
+    """
     retrieve:
         Return a execute template instance.
 
@@ -574,20 +559,24 @@ class __TemplateViewSet(base.ModelViewSet):
 
     update:
         Update a execute template.
-    '''
+    """
     model = sers.models.Template
     serializer_class = sers.TemplateSerializer
     serializer_class_one = sers.OneTemplateSerializer
-    filter_class = filters.TemplateFilter
+    filterset_class = filters.TemplateFilter
     POST_WHITE_LIST = ['execute']
 
-    @deco.subaction(serializer_class=sers.TemplateExecSerializer, **execute_kw)
+    @deco.subaction(**{
+        **execute_kw,
+        'serializer_class': sers.TemplateExecSerializer,
+        'response_serializer': sers.ExecuteResponseSerializer
+    })
     def execute(self, request, *args, **kwargs):
-        '''
+        """
         Execute template with option.
-        '''
-        obj = self.get_object()
-        return self.get_serializer(obj).execute(request)
+        """
+        # returns HTTPResponse
+        return self.get_object().execute(request.user, request.data.get('option', None))
 
 
 class __ProjectHistoryViewSet(HistoryViewSet):
@@ -602,7 +591,7 @@ class __ProjectHistoryViewSet(HistoryViewSet):
 @deco.nested_view('history', 'id', manager_name='history', view=__ProjectHistoryViewSet)
 @deco.nested_view('variables', 'id', view=__ProjectVarsViewSet)
 class ProjectViewSet(OwnedView, _VariablesCopyMixin):
-    '''
+    """
     retrieve:
         Return a project instance.
 
@@ -620,51 +609,108 @@ class ProjectViewSet(OwnedView, _VariablesCopyMixin):
 
     update:
         Update a project.
-    '''
+    """
     model = sers.models.Project
     serializer_class = sers.ProjectSerializer
     serializer_class_one = sers.OneProjectSerializer
     serializer_class_create = sers.ProjectCreateMasterSerializer
-    filter_class = filters.ProjectFilter
+    filterset_class = filters.ProjectFilter
     POST_WHITE_LIST = ['sync', 'execute_playbook', 'execute_module']
     copy_related = ['inventories']
 
     def copy_instance(self, instance):
         instance.status = instance.__class__._meta.get_field('status').default
-        return super(ProjectViewSet, self).copy_instance(instance)
+        return super().copy_instance(instance)
 
     @deco.subaction(serializer_class=vstsers.EmptySerializer, **action_kw)
+    @transaction.atomic()
     def sync(self, request, *args, **kwargs):
-        '''
+        """
         Sync project with repository.
-        '''
-        return self.get_serializer(self.get_object()).sync().resp
+        """
+        instance = self.get_object()
+        instance.start_repo_task("sync")
+        serializer = sers.ActionResponseSerializer(
+            data=dict(detail=f"Sync with {instance.repository}.")
+        )
+        serializer.is_valid(True)
+        return HTTP_200_OK(serializer.data)
 
-    @deco.subaction(serializer_class=sers.AnsiblePlaybookSerializer, **execute_kw)
+    @deco.subaction(**{
+        **execute_kw,
+        'serializer_class': sers.AnsiblePlaybookSerializer,
+        'response_serializer': sers.ExecuteResponseSerializer
+    })
     def execute_playbook(self, request, *args, **kwargs):
-        '''
+        """
         Execute `ansible-playbook` with arguments.
-        '''
-        serializer = self.get_serializer(self.get_object())
-        return serializer.execute_playbook(request).resp
+        """
+        return self._execution("playbook", dict(request.data), request.user)
 
-    @deco.subaction(serializer_class=sers.AnsibleModuleSerializer, **execute_kw)
+    @deco.subaction(**{
+        **execute_kw,
+        'serializer_class': sers.AnsibleModuleSerializer,
+        'response_serializer': sers.ExecuteResponseSerializer
+    })
     def execute_module(self, request, *args, **kwargs):
-        '''
+        """
         Execute `ansible -m [module]` with arguments.
-        '''
-        serializer = self.get_serializer(self.get_object())
-        return serializer.execute_module(request).resp
+        """
+        return self._execution("module", dict(request.data), request.user)
+
+    def _execution(self, kind, data, user, **kwargs):
+
+        template = data.pop("template", None)
+        inventory = data.get("inventory", None)
+        msg = "Started in the inventory {}.".format(
+            inventory if inventory else 'specified in the project configuration.'
+        )
+        instance = self.get_object()
+        if template is not None:
+            init_type = "template"
+            obj_id = template
+            msg = f'Start template [id={template}].'
+        else:
+            init_type = "project"
+            obj_id = instance.id
+            serializer = self._get_ansible_serializer(kind.lower())
+            data = {
+                k: v for k, v in serializer.to_internal_value(data).items()
+                if k in data.keys() or v
+            }
+        target = data.pop(kind)
+        try:
+            target = str(target)
+        except UnicodeEncodeError:  # nocv
+            target = target.encode('utf-8')
+        history_id = instance.execute(
+            kind, str(target),
+            initiator=obj_id, initiator_type=init_type, executor=user, **data
+        )
+        rdata = sers.ExecuteResponseSerializer(data=dict(
+            detail=msg,
+            history_id=history_id, executor=user.id
+        ))
+        rdata.is_valid(raise_exception=True)
+        return HTTP_201_CREATED(rdata.data)
+
+    def _get_ansible_serializer(self, kind):
+        exec_method = getattr(self, 'execute_{}'.format(kind), None)
+        if exec_method is None:  # nocv
+            raise Exception('Unknown kind')
+        serializer: sers.serializers.Serializer = self.get_serializer()
+        serializer.project = self.get_object()
+        return serializer
 
 
 class ProjectTemplateViewSet(base.ReadOnlyModelViewSet):
-    '''
+    """
     retrieve:
         Return a community project template instance.
 
     list:
         List of community project templates.
-    '''
+    """
     model = sers.models.ProjectTemplate
     serializer_class = sers.ProjectTemplateSerializer
     serializer_class_one = sers.OneProjectTemplateSerializer
@@ -676,17 +722,17 @@ class ProjectTemplateViewSet(base.ReadOnlyModelViewSet):
         **default_action
     )
     def use_it(self, request, *args, **kwargs):
-        '''
+        """
         Create project based on this template.
-        '''
+        """
         serializer = self.get_serializer(self.get_object(), data=request.data)
         serializer.is_valid(raise_exception=True)
         serializer.save()
-        return base.Response(serializer.data, status=status.HTTP_201_CREATED).resp
+        return HTTP_201_CREATED(serializer.data)
 
 
 class HookViewSet(base.ModelViewSet):
-    '''
+    """
     retrieve:
         Return a hook instance.
 
@@ -704,10 +750,10 @@ class HookViewSet(base.ModelViewSet):
 
     update:
         Update a hook.
-    '''
+    """
     model = sers.models.Hook
     serializer_class = sers.HookSerializer
-    filter_class = filters.HookFilter
+    filterset_class = filters.HookFilter
     permission_classes = (StaffPermission,)
 
 
@@ -718,31 +764,30 @@ class HookViewSet(base.ModelViewSet):
 class StatisticViewSet(base.ListNonModelViewSet):
     base_name = "stats"
 
-    def _get_count_by_user(self, model):
+    def _get_by_user(self, model):
         user = self.request.user
         filter_models = (sers.User,)
         if model not in filter_models:
-            return model.objects.all().user_filter(user).count()
-        return model.objects.all().count()
+            return model.objects.all().user_filter(user)
+        return model.objects.all()
 
     def _get_history_stats(self, request):
         qs = sers.models.History.objects.all()
         qs = qs.user_filter(self.request.user)
         return qs.stats(int(request.query_params.get("last", "14")))
 
+    def _get_by_user_projects(self, model):
+        return model.objects.filter(project__in=self._get_by_user(sers.models.Project).values('id'))
+
     def list(self, request, *args, **kwargs):
         # pylint: disable=unused-argument
         stats = OrderedDict()
-        stats['projects'] = self._get_count_by_user(sers.models.Project)
-        stats['templates'] = self._get_count_by_user(sers.models.Template)
-        stats['inventories'] = self._get_count_by_user(sers.models.Inventory)
-        stats['groups'] = self._get_count_by_user(sers.models.Group)
-        stats['hosts'] = self._get_count_by_user(sers.models.Host)
-        stats['teams'] = self._get_count_by_user(sers.models.UserGroup)
-        stats['users'] = self._get_count_by_user(sers.User)
+        stats['projects'] = self._get_by_user(sers.models.Project).count()
+        stats['templates'] = self._get_by_user_projects(sers.models.Template).count()
+        stats['inventories'] = self._get_by_user(sers.models.Inventory).count()
+        stats['groups'] = self._get_by_user(sers.models.Group).count()
+        stats['hosts'] = self._get_by_user(sers.models.Host).count()
+        stats['teams'] = self._get_by_user(sers.models.UserGroup).count()
+        stats['users'] = self._get_by_user(sers.User).count()
         stats['jobs'] = self._get_history_stats(request)
-        return base.Response(stats, status.HTTP_200_OK).resp
-
-
-class BulkViewSet(views.BulkViewSet):
-    __doc__ = views.BulkViewSet.__doc__
+        return HTTP_200_OK(stats)
