@@ -4,6 +4,7 @@ from typing import Any, Text, Iterable, Dict, Union
 import sys
 import json
 import logging
+from itertools import chain
 from collections import OrderedDict
 from django_celery_beat.models import IntervalSchedule, CrontabSchedule, PeriodicTask as CPTask
 from django.db.models import signals, IntegerField
@@ -12,13 +13,14 @@ from django.dispatch import receiver
 from django.db.models.functions import Cast
 from django.core.validators import ValidationError
 from django.conf import settings
+from rest_framework.exceptions import ValidationError as drfValidationError
 from vstutils.utils import raise_context, KVExchanger
 
 from .vars import Variable
 from .hosts import Host, Group, Inventory
 from .projects import Project, Task, Module, ProjectTemplate, list_to_choices
-from .users import get_user_model, UserGroup, ACLPermission, UserSettings
-from .tasks import PeriodicTask, History, HistoryLines, Template
+from .users import get_user_model, UserGroup, ACLPermission
+from .tasks import PeriodicTask, History, HistoryLines, Template, TemplateOption, ExecutionTypes
 from .hooks import Hook
 from ..validators import RegexValidator, validate_hostname, path_validator
 from ..exceptions import UnknownTypeException, Conflict
@@ -132,11 +134,11 @@ def check_circular_deps(instance: Group, action: Text, pk_set: Iterable, *args,
         return
     if (action in ["pre_add", "post_add"]) and ('loaddata' not in sys.argv):
         if instance.id in pk_set:
-            raise instance.CiclicDependencyError("The group can not refer to itself.")
+            raise instance.CyclicDependencyError("The group can not refer to itself.")
         parents = instance.parents.all().get_parents()
-        childrens = instance.groups.all().get_subgroups()
-        if instance in (parents | childrens) or parents.filter(id__in=pk_set).count():
-            raise instance.CiclicDependencyError("The group has a dependence on itself.")
+        children = instance.groups.all().get_children()
+        if instance in (parents | children) or parents.filter(id__in=pk_set).count():
+            raise instance.CyclicDependencyError("The group has a dependence on itself.")
 
 
 @receiver(signals.pre_save, sender=PeriodicTask)
@@ -156,7 +158,7 @@ def validate_crontab(instance: PeriodicTask, **kwargs) -> None:
         instance.get_schedule()
     except ValueError as ex:
         msg = dict(schedule=["{}".format(ex)])
-        raise ValidationError(msg)
+        raise ValidationError(msg) from ex
 
 
 @receiver(signals.pre_save, sender=Host)
@@ -174,24 +176,22 @@ def validate_type_and_name(instance: Host, **kwargs) -> None:
 
 @receiver(signals.pre_save, sender=Template)
 def validate_template_keys(instance: Template, **kwargs) -> None:
-    if 'loaddata' in sys.argv or kwargs.get('raw', False):  # noce
+    if 'loaddata' in sys.argv or kwargs.get('raw', False):  # nocv
         return
-    # Deprecated, because moved to serializers
-    if instance.kind not in instance.template_fields.keys():
-        raise UnknownTypeException(instance.kind)
     errors = {}
-    for key in instance.data.keys():
-        if key not in instance.template_fields[instance.kind]:
-            errors[key] = "Unknown key. Keys should be {}".format(
-                instance.template_fields[instance.kind]
-            )
+    for _, data in chain(((None, instance.data),), instance.options.items()):
+        for key in data.keys():
+            if key not in instance.template_fields[instance.kind]:
+                errors[key] = ["Unknown key. Keys should be {}".format(
+                    instance.template_fields[instance.kind]
+                )]
     if errors:
-        raise ValidationError(errors)
+        raise drfValidationError(errors)
 
 
 @receiver(signals.pre_save, sender=Template)
 def validate_template_args(instance: Template, **kwargs) -> None:
-    if 'loaddata' in sys.argv or kwargs.get('raw', False):  # noce
+    if 'loaddata' in sys.argv or kwargs.get('raw', False):  # nocv
         return
     if instance.kind in ["Host", "Group"]:
         return  # nocv
@@ -234,7 +234,7 @@ def save_to_beat(instance: PeriodicTask, **kwargs) -> None:
     :param kwargs:
     :return:
     """
-    if 'loaddata' in sys.argv or kwargs.get('raw', False):  # noce
+    if 'loaddata' in sys.argv or kwargs.get('raw', False):  # nocv
         return
 
     if not instance.enabled:
@@ -249,7 +249,7 @@ def save_to_beat(instance: PeriodicTask, **kwargs) -> None:
 
     # Try get Celery Periodic Task, that linked with Polemarch Periodic Task
     celery_task = CPTask.objects.filter(
-        name=str(instance.id), task=settings.TASKS_HANDLERS["SCHEDUER"]["BACKEND"]
+        name=str(instance.id), task=settings.TASKS_HANDLERS["SCHEDULER"]["BACKEND"]
     ).last()
 
     # Prepare data for Schedule
@@ -296,7 +296,7 @@ def save_to_beat(instance: PeriodicTask, **kwargs) -> None:
         # Create new Celery Periodic Task, if it doesn't  exist
         CPTask.objects.create(
             name=str(instance.id),
-            task=settings.TASKS_HANDLERS["SCHEDUER"]["BACKEND"],
+            task=settings.TASKS_HANDLERS["SCHEDULER"]["BACKEND"],
             args=json.dumps([instance.id]),
             **{instance_pt_type: types_dict[instance_pt_type].objects.get_or_create(**schedule_data)[0]}
         )
@@ -316,7 +316,7 @@ def delete_from_beat(instance: PeriodicTask, **kwargs) -> None:
 
     # Get Celery Periodic Task
     celery_task = CPTask.objects.filter(
-        name=str(instance.id), task=settings.TASKS_HANDLERS["SCHEDUER"]["BACKEND"]
+        name=str(instance.id), task=settings.TASKS_HANDLERS["SCHEDULER"]["BACKEND"]
     ).last()
 
     if celery_task:
@@ -349,7 +349,6 @@ def check_if_inventory_linked(instance: Inventory, action: Text, **kwargs) -> No
             ),
         )
 
-
 @receiver(signals.pre_delete, sender=Inventory)
 def delete_imported_file_inventories(instance: Inventory, **kwargs) -> None:
     # pylint: disable=invalid-name
@@ -372,7 +371,7 @@ def check_if_inventory_linked_project(instance: Inventory, **kwargs) -> None:
 
 @receiver(signals.pre_save, sender=Hook)
 def check_hook(instance: Hook, **kwargs) -> None:
-    if 'loaddata' in sys.argv or kwargs.get('raw', False):  # noce
+    if 'loaddata' in sys.argv or kwargs.get('raw', False):  # nocv
         return
     errors = instance.handlers.validate(instance)
     if errors:
@@ -403,7 +402,7 @@ def user_add_hook(instance: BaseUser, **kwargs) -> None:
 @receiver([signals.post_save, signals.post_delete], sender=Group)
 @receiver([signals.post_save, signals.post_delete], sender=Host)
 def polemarch_hook(instance: Any, **kwargs) -> None:
-    if 'loaddata' in sys.argv or kwargs.get('raw', False):  # noce
+    if 'loaddata' in sys.argv or kwargs.get('raw', False):  # nocv
         return
     created = kwargs.get('created', None)
     when = "on_object_add"
@@ -419,16 +418,9 @@ def polemarch_hook(instance: Any, **kwargs) -> None:
     send_polemarch_models(when, instance)
 
 
-@receiver(signals.post_save, sender=BaseUser)
-def create_settings_for_user(instance: BaseUser, **kwargs) -> None:
-    if 'loaddata' in sys.argv or kwargs.get('raw', False):  # nocv
-        return
-    UserSettings.objects.get_or_create(user=instance)
-
-
 @receiver(signals.pre_save, sender=Template)
 def update_ptasks_with_templates(instance: Template, **kwargs) -> None:
-    if 'loaddata' in sys.argv or kwargs.get('raw', False):  # noce
+    if 'loaddata' in sys.argv or kwargs.get('raw', False):  # nocv
         return
     instance.periodic_task.all().update(project=instance.project)
 
@@ -441,6 +433,7 @@ def cancel_task_on_delete_history(instance: History, **kwargs) -> None:
 
 @receiver(signals.post_migrate)
 def update_crontab_timezone_ptasks(*args, **kwargs):
+    # pylint: disable=no-member
     qs = CrontabSchedule.objects.exclude(timezone=settings.TIME_ZONE)
     qs.filter(periodictask__task__startswith='polemarch').update(timezone=settings.TIME_ZONE)
     qs.filter(periodictask__task__startswith='pmlib').update(timezone=settings.TIME_ZONE)
