@@ -1,19 +1,20 @@
 # pylint: disable=protected-access,no-member
 from __future__ import unicode_literals
-from typing import NoReturn, Any, Dict, List, Tuple, Iterable, TypeVar, Generator, Text
+
+from typing import NoReturn, Any, Dict, List, Tuple, Iterable, TypeVar, Text
 import logging
 from collections import OrderedDict
 from datetime import timedelta, datetime
-from functools import partial
 import json
 
 import re
-import io
 from celery.schedules import crontab
 from django.core.exceptions import ValidationError
+from django.db.utils import IntegrityError
 from django.db import transaction
 from django.db.models import Q
 from django.utils import timezone
+from django.utils.text import slugify
 from django.contrib.auth import get_user_model
 from django.db.models import functions as dbfunc, Count
 from django.utils.timezone import now
@@ -21,11 +22,14 @@ from django.test import Client
 from django.conf import settings
 from rest_framework.exceptions import UnsupportedMediaType
 
+from vstutils.custom_model import ListModel, CustomQuerySet
+from vstutils.utils import BaseEnum, translate as _
 from . import Inventory
 from ..exceptions import DataNotReady, NotApplicable
-from .base import ForeignKeyACL, BModel, ACLModel, BQuerySet, models
+from .base import ForeignKeyACL, BModel, ACLModel, BQuerySet, models, BaseModel
 from .vars import AbstractModel, AbstractVarsQuerySet
 from .projects import Project, HISTORY_ID
+from ..constants import CYPHER, HiddenArg
 
 
 logger = logging.getLogger("polemarch")
@@ -33,6 +37,12 @@ InvOrString = TypeVar('InvOrString', str, int, Inventory, None)
 User = get_user_model()
 
 
+class ExecutionTypes(BaseEnum):
+    # pylint: disable=invalid-name
+    Task = 'Task'
+    Module = 'Module'
+
+
 # Block of real models
 class Template(ACLModel):
     name          = models.CharField(max_length=512)
@@ -69,7 +79,10 @@ class Template(ACLModel):
         data = json.loads(self.template_data)
         if "inventory" in self.template_fields[self.kind] and self.inventory:
             try:
-                data['inventory'] = int(self.inventory)
+                if isinstance(self.inventory, Inventory):
+                    data['inventory'] = self.inventory.id
+                else:
+                    data['inventory'] = int(self.inventory)
             except ValueError:
                 data['inventory'] = self.inventory
         return data
@@ -127,7 +140,7 @@ class Template(ACLModel):
 
     def __encrypt(self, new_vars: Dict, data_name: Text = 'data') -> Dict:
         old_vars = getattr(self, data_name).get('vars', {})
-        for key in filter(lambda k: new_vars[k] == '[~~ENCRYPTED~~]', new_vars.keys()):
+        for key in filter(lambda k: new_vars[k] == CYPHER, new_vars.keys()):
             new_vars[key] = old_vars.get(key, new_vars[key])
         return new_vars
 
@@ -147,7 +160,7 @@ class Template(ACLModel):
 
     def set_options_data(self, value: Any) -> NoReturn:
         options_data = self._convert_to_data(value)
-        new = dict()
+        new = {}
         for option, data in options_data.items():
             self._validate_option_data(data)
             if data.get('vars', None):
@@ -196,6 +209,63 @@ class Template(ACLModel):
     def options_list(self) -> List[str]:
         return list(self.options.keys())
 
+    @property
+    def options_qs(self):
+        return TemplateOption.objects.prepare_related(self)
+
+
+class TemplateOptionQueryset(CustomQuerySet):
+    def prepare_related(self, template):
+        return self.setup_custom_queryset_kwargs(template=template)._chain(
+            field=[f for f in self.model._meta.fields if f.name == 'template'][0],
+            instance=template,
+        )
+
+
+class TemplateOption(ListModel):
+    objects = TemplateOptionQueryset.as_manager()
+
+    template = models.ForeignKey(Template, on_delete=models.DO_NOTHING)
+    id = models.SlugField(max_length=256, primary_key=True)
+    name = models.CharField(max_length=256)
+    data = models.JSONField(default=dict)
+
+    class Meta:
+        managed = False
+
+    def get_pk_value(self):
+        return slugify(self.name)
+
+    def set_pk_value(self, value):
+        pass
+
+    @classmethod
+    def _get_data(cls, template: Template, **kwargs):
+        return [
+            {'id': slugify(key), 'name': key, 'data': value, 'template': template}
+            for key, value in template.options.items()
+        ]
+
+    def save(self, **kwargs):
+        # TODO: Make this code transaction atomic
+        if kwargs.get('force_insert', False) and slugify(self.name) in map(slugify, self.template.options.keys()):
+            raise IntegrityError(_('Template option name should be unique'))
+        self.template.set_options_data({
+            **self.template.options,
+            self.name: self.data,
+        })
+        self.template.save()
+
+    def delete(self, **kwargs):
+        options = self.template.options
+        del options[self.name]
+        self.template.set_options_data(options)
+        self.template.save()
+
+    @property
+    def kind(self):
+        return self.template.kind
+
 
 class PeriodicTaskQuerySet(AbstractVarsQuerySet):
     use_for_related_fields = True
@@ -223,16 +293,6 @@ class PeriodicTask(AbstractModel):
 
     kinds = ["PLAYBOOK", "MODULE", "TEMPLATE"]
     types = ["CRONTAB", "INTERVAL"]
-    HIDDEN_VARS = [
-        'key-file',
-        'key_file',
-        'private-key',
-        'private_key',
-        'vault-password-file',
-        'vault_password_file',
-        'new-vault-password-file',
-        'new_vault_password_file',
-    ]
 
     class Meta:
         default_related_name = "periodic_task"
@@ -253,21 +313,16 @@ class PeriodicTask(AbstractModel):
 
     @inventory.setter
     def inventory(self, inventory: InvOrString) -> NoReturn:
-        if isinstance(inventory, Inventory):  # nocv
+        if isinstance(inventory, Inventory):
             self._inventory = inventory
             self.inventory_file = None
-        elif isinstance(inventory, (str, int)):
-            try:
-                self._inventory = self.project.inventories.get(pk=int(inventory))
-                self.inventory_file = None
-            except (ValueError, Inventory.DoesNotExist):
-                self.project.check_path(inventory)
-                self.inventory_file = inventory
-                self._inventory = None
+        elif isinstance(inventory, str):
+            self.inventory_file = inventory
+            self._inventory = None
 
     @property
     def crontab_kwargs(self) -> Dict:
-        kwargs, index, fields = dict(), 0, self.schedule.split(" ")
+        kwargs, index, fields = {}, 0, self.schedule.split(" ")
         for field_name in self.time_types_list:
             if index < len(fields) and len(fields[index]) > 0:
                 kwargs[field_name] = fields[index]
@@ -311,7 +366,7 @@ class HistoryQuerySet(BQuerySet):
 
     def create(self, **kwargs) -> BModel:
         raw_stdout = kwargs.pop("raw_stdout", None)
-        history = super(HistoryQuerySet, self).create(**kwargs)
+        history = super().create(**kwargs)
         if raw_stdout:
             history.raw_stdout = raw_stdout
         return history
@@ -329,7 +384,7 @@ class HistoryQuerySet(BQuerySet):
         return values
 
     def stats(self, last: int) -> OrderedDict:
-        qs = self.filter(start_time__gte=now()-timedelta(days=last))
+        qs = self.filter(start_time__gte=timezone.now()-timedelta(days=last))
         qs = qs.annotate(
             day=dbfunc.TruncDay('start_time'),
             month=dbfunc.TruncMonth('start_time'),
@@ -345,7 +400,7 @@ class HistoryQuerySet(BQuerySet):
         return {opt: extra.pop(opt, options[opt]) for opt in options}
 
     def __get_additional_options(self, extra_options) -> Dict[str, Any]:
-        options = dict()
+        options = {}
         if extra_options['template_option'] is not None:
             options['template_option'] = extra_options['template_option']
         return options
@@ -400,7 +455,7 @@ class History(BModel):
 
     def __init__(self, *args, **kwargs):
         execute_args = kwargs.pop('execute_args', None)
-        super(History, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         if execute_args:
             self.execute_args = execute_args
 
@@ -450,9 +505,7 @@ class History(BModel):
         if not isinstance(value, dict):
             raise ValidationError(dict(args="Should be a dict."))
         data = {k: v for k, v in value.items() if k not in ['group']}
-        for key in PeriodicTask.HIDDEN_VARS:
-            if key in data:
-                data[key] = "[~~ENCRYPTED~~]"
+        HiddenArg.hide_values(data)
         self.json_args = json.dumps(data)
 
     # options
@@ -494,6 +547,7 @@ class History(BModel):
         )
         subst = '"\\1": {\n\t"status": "\\2", \n\\3},'
         result = re.sub(regex, subst, data, 0, re.MULTILINE)
+        result = re.findall(r'^".*":[\s\S]*$', result, re.MULTILINE)[0]
         result = "{" + result[:-1] + "\n}"
         return json.loads(result)
 
@@ -527,32 +581,18 @@ class History(BModel):
             counter += 1
             self.write_line(number=counter, value=line)
 
-    def __create_line(self, gnum: int, num: int, val: str, hidden: bool = False) -> BModel:
-        return HistoryLines(
-            history=self, line_gnumber=gnum, line_number=num, line=val, hidden=hidden
-        )
-
-    def __bulking_lines(self, value: str, number: int, endl: str) -> Generator:
-        out = io.StringIO(value)
-        nline = 0
-        for line in iter(partial(out.read, 2 * 1024 - 100), ''):
-            nline += 1
-            yield self.__create_line(number, nline, line)
-        if endl:
-            yield self.__create_line(number, nline, endl)
-
     def write_line(self, value: str, number: int, endl: Text = "") -> NoReturn:
-        self.raw_history_line.bulk_create(
-            map(lambda l: l, self.__bulking_lines(value, number, endl))
-        )
+        self.raw_history_line.bulk_create([
+            HistoryLines(line_gnumber=number, line_number=1, line=value + endl, history=self)
+        ])
 
 
-class HistoryLines(BModel):
-    line         = models.CharField(default="", max_length=2*1024)
-    line_number  = models.IntegerField(default=0)
+class HistoryLines(BaseModel):
+    id = models.BigAutoField(primary_key=True)
+    line = models.TextField(default="")
+    line_number = models.IntegerField(default=0)
     line_gnumber = models.IntegerField(default=0)
-    history      = models.ForeignKey(History, on_delete=models.CASCADE,
-                                     related_query_name="raw_history_line")
+    history = models.ForeignKey(History, on_delete=models.CASCADE, related_query_name="raw_history_line")
 
     class Meta:
         default_related_name = "raw_history_line"
