@@ -1,8 +1,7 @@
 # pylint: disable=protected-access,no-member
 from __future__ import unicode_literals
-from typing import NoReturn, Dict, Text
+from typing import NoReturn, Text
 import logging
-import json
 
 from django.contrib.auth.models import Group as BaseGroup
 from django.contrib.auth import get_user_model
@@ -45,25 +44,3 @@ class UserGroup(BaseGroup, ACLModel):
     objects = BQuerySet.as_manager()
     parent = models.OneToOneField(BaseGroup, on_delete=models.CASCADE, parent_link=True)
     users = BaseGroup.user_set
-
-
-class UserSettings(BModel):
-    settings = models.TextField(default="{}")
-    user     = models.OneToOneField(get_user_model(), on_delete=models.CASCADE,
-                                    related_query_name="settings",
-                                    related_name="settings")
-
-    def get_settings_copy(self) -> Dict:
-        return self.data
-
-    @property
-    def data(self) -> Dict:
-        return json.loads(self.settings)
-
-    @data.setter
-    def data(self, value) -> NoReturn:
-        self.settings = json.dumps(value)
-
-    @data.deleter
-    def data(self) -> NoReturn:
-        self.settings = '{}'
