@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
+import threading
 from typing import NoReturn, Text, Any, Iterable, Tuple, List, Dict, Union
 import os
 import re
@@ -13,12 +14,15 @@ from pathlib import Path
 from collections import namedtuple, OrderedDict
 from subprocess import Popen
 from functools import reduce
+from django.apps import apps
 from django.utils import timezone
 from vstutils.utils import tmp_file, KVExchanger, raise_context
 from vstutils.tools import get_file_value
+
 from .hosts import Inventory
 from .tasks import History, Project
 from ...main.utils import CmdExecutor, AnsibleArgumentsReference, PMObject
+from ..constants import HiddenArg, HiddenVar, CYPHER
 
 
 logger = logging.getLogger("polemarch")
@@ -63,10 +67,10 @@ class DummyHistory:
 
 
 class Executor(CmdExecutor):
-    __slots__ = 'history', 'counter', 'exchanger'
+    __slots__ = 'history', 'counter', 'exchanger', 'notificator',  'notificator_lock', 'notification_last_time'
 
     def __init__(self, history: History):
-        super(Executor, self).__init__()
+        super().__init__()
         self.history = history
         self.counter = 0
         self.exchanger = KVExchanger(self.CANCEL_PREFIX + str(self.history.id))
@@ -74,6 +78,13 @@ class Executor(CmdExecutor):
         if self.history.project is not None:
             env_vars = self.history.project.env_vars
         self.env = env_vars
+        if isinstance(history, DummyHistory):
+            self.notificator = None
+        else:
+            notificator_class = apps.get_app_config('vstutils_api').module.notificator_class
+            self.notificator = notificator_class([], channel='history_lines')
+            self.notificator_lock = threading.Lock()
+            self.notification_last_time = 0
 
     @property
     def output(self) -> Text:
@@ -98,22 +109,36 @@ class Executor(CmdExecutor):
             proc.terminate()
             proc.kill()
             proc.wait()
-        super(Executor, self).working_handler(proc)
+        super().working_handler(proc)
+        if self.notificator and time.time() - self.notification_last_time > 1:
+            with self.notificator_lock:
+                if self.notificator.queue:
+                    self.notification_last_time = time.time()
+                    self.notificator.send()
 
     def write_output(self, line: Text):
         self.counter += 1
         self.history.write_line(line, self.counter, '\n')
+        if self.notificator:
+            with self.notificator_lock:
+                self.notificator.create_notification_from_instance(self.history)
 
     def execute(self, cmd: Iterable[Text], cwd: Text):
         pm_ansible_path = ' '.join(self.pm_ansible())
-        new_cmd = list()
+        new_cmd = []
         for one_cmd in cmd:
             if isinstance(one_cmd, str):
                 with raise_context():
                     one_cmd = one_cmd.decode('utf-8')
             new_cmd.append(one_cmd)
         self.history.raw_args = " ".join(new_cmd).replace(pm_ansible_path, '').lstrip()
-        return super(Executor, self).execute(new_cmd, cwd)
+        ret = super().execute(new_cmd, cwd)
+        if self.notificator:
+            self.notificator.disconnect_all()
+            with self.notificator_lock:
+                if self.notificator.queue:
+                    self.notificator.send()
+        return ret
 
 
 class AnsibleCommand(PMObject):
@@ -137,8 +162,6 @@ class AnsibleCommand(PMObject):
         '''
 
     class Inventory(object):
-        hidden_vars = Inventory.HIDDEN_VARS
-
         def __init__(self, inventory: Union[Inventory, int, Text], cwd: Text = "/tmp", tmpdir: Text = '/tmp'):
             self.cwd = cwd
             self.tmpdir = tmpdir
@@ -197,7 +220,7 @@ class AnsibleCommand(PMObject):
         self.verbose = kwargs.get('verbose', 0)
         self.cwd = tempfile.mkdtemp()
         self._verbose_output('Execution tmpdir created - [{}].'.format(self.cwd), 0)
-        self.env = dict()
+        self.env = {}
 
     def _verbose_output(self, value: Text, level: int = 3) -> NoReturn:
         if self.verbose >= level:
@@ -244,10 +267,10 @@ class AnsibleCommand(PMObject):
         if key == 'verbose':
             extra_args += ['-' + ('v' * value)] if value else []
             return extra_args, files
-        result = [value, list()]
-        if key in ["key-file", "private-key"]:
+        result = [value, []]
+        if key in HiddenArg.get_text_values():
             result = self.__parse_key(key, value)
-        elif key in ["vault-password-file", "new-vault-password-file"]:
+        elif key in HiddenArg.get_file_values():
             result = self.__generate_arg_file(value)  # nocv
         value = result[0]
         files += result[1]
@@ -272,9 +295,6 @@ class AnsibleCommand(PMObject):
     def path_to_ansible(self) -> List[Text]:
         return self.pm_ansible(self.command_type)
 
-    def get_hidden_vars(self) -> List[Text]:
-        return self.inventory_object.hidden_vars
-
     def get_inventory_arg(self, target: Text, extra_args: List[Text]) -> List[Text]:
         # pylint: disable=unused-argument
         args = [target]
@@ -299,10 +319,9 @@ class AnsibleCommand(PMObject):
     def hide_passwords(self, raw: Text) -> Text:
         regex = r'|'.join((
             r"(?<=" + hide + r":\s).{1,}?(?=[\n\t\s])"
-            for hide in self.get_hidden_vars()
+            for hide in HiddenVar.get_values()
         ))
-        subst = "[~~ENCRYPTED~~]"
-        raw = re.sub(regex, subst, raw, 0, re.MULTILINE)
+        raw = re.sub(regex, CYPHER, raw, 0, re.MULTILINE)
         return raw
 
     def get_execution_revision(self, project: Project):  # nocv
@@ -341,7 +360,7 @@ class AnsibleCommand(PMObject):
 
     def dir_prepare_git(self, src: Text, work_dir: Text, revision: Text):
         # pylint: disable=no-member
-        import git
+        import git  # pylint: disable=import-outside-toplevel
         repo = git.Repo.clone_from(
             url=src + "/.git",
             to_path=work_dir,
@@ -436,8 +455,8 @@ class AnsibleModule(AnsibleCommand):
         kwargs['module-name'] = target
         if not kwargs.get('args', None):
             kwargs.pop('args', None)
-        super(AnsibleModule, self).__init__(*pargs, **kwargs)
+        super().__init__(*pargs, **kwargs)
         self.ansible_ref['module-name'] = {'type': 'string'}
 
     def execute(self, group: Text = 'all', *args, **extra_args):
-        return super(AnsibleModule, self).execute(group, *args, **extra_args)
+        return super().execute(group, *args, **extra_args)
