@@ -12,7 +12,7 @@ from django.contrib.contenttypes.models import ContentType
 from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
 from vstutils.utils import tmp_file
 from .base import ACLModel, BQuerySet, BModel, models
-
+from ..constants import CYPHER
 
 logger = logging.getLogger("polemarch")
 
@@ -32,7 +32,7 @@ class VariablesQuerySet(BQuerySet):
     use_for_related_fields = True
 
     def sort_by_key(self) -> BQuerySet:
-        args, kwargs = [], dict()
+        args, kwargs = [], {}
         keys = self.model.variables_keys
         index = keys.index
         for key in keys:
@@ -43,16 +43,16 @@ class VariablesQuerySet(BQuerySet):
         return self.annotate(sort_idx=Case(*args, **kwargs)).order_by("sort_idx", "key")
 
     def cleared(self) -> BQuerySet:
-        return super(VariablesQuerySet, self).cleared().sort_by_key()
+        return super().cleared().sort_by_key()
 
 
 class Variable(BModel):
     objects = VariablesQuerySet.as_manager()
-    content_type   = models.ForeignKey(ContentType, on_delete=models.CASCADE)
-    object_id      = models.PositiveIntegerField()
+    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
+    object_id = models.PositiveIntegerField()
     content_object = GenericForeignKey('content_type', 'object_id')
-    key            = models.CharField(max_length=512)
-    value          = models.TextField(null=True)
+    key = models.CharField(max_length=512)
+    value = models.TextField(null=True)
 
     variables_keys = [
         "ansible_host",
@@ -98,22 +98,13 @@ class AbstractVarsQuerySet(BQuerySet):
 
 
 class AbstractModel(ACLModel):
-    objects     = AbstractVarsQuerySet.as_manager()
-    name        = models.CharField(max_length=512, default=uuid.uuid1)
-    variables   = GenericRelation(Variable, related_query_name="variables",
-                                  object_id_field="object_id")
+    objects = AbstractVarsQuerySet.as_manager()
+    name = models.CharField(max_length=512, default=uuid.uuid1)
+    variables = GenericRelation(Variable, related_query_name="variables", object_id_field="object_id")
 
     class Meta:
         abstract = True
 
-    HIDDEN_VARS = [
-        'ansible_ssh_pass',
-        'ansible_ssh_private_key_file',
-        'ansible_become_pass',
-        'ansible_become_password',
-        'ansible_password'
-    ]
-
     BOOLEAN_VARS = []
 
     def __unicode__(self):  # pragma: no cover
@@ -130,9 +121,8 @@ class AbstractModel(ACLModel):
 
     @transaction.atomic()
     def set_vars(self, variables) -> NoReturn:
-        encr = "[~~ENCRYPTED~~]"
-        encrypted_vars = {k: v for k, v in variables.items() if v == encr}
-        other_vars = {k: v for k, v in variables.items() if v != encr}
+        encrypted_vars = {k: v for k, v in variables.items() if v == CYPHER}
+        other_vars = {k: v for k, v in variables.items() if v != CYPHER}
         self.variables.cleared().exclude(key__in=list(encrypted_vars.keys()) + list(other_vars.keys())).delete()
         for key, value in other_vars.items():
             self.variables.create(key=key, value=value)
@@ -142,7 +132,7 @@ class AbstractModel(ACLModel):
         return reduce(update_boolean, self.BOOLEAN_VARS, OrderedDict(qs))
 
     def get_vars_prefixed(self, prefix: Text):
-        vars_by_prefix_dict = dict()
+        vars_by_prefix_dict = {}
         search_prefix = prefix + '_'
         search_prefix_len = len(search_prefix)
         for var_obj in self.variables.filter(key__startswith=prefix):
