@@ -110,6 +110,7 @@ class _Base:
         :param data: all data from file
         '''
         # pylint: disable=unused-argument
+        self.proj.get_yaml_subcache().clear()
         self.proj.get_yaml_subcache('view').clear()
         self.message(self.proj.execute_view_data, 'debug')
 
@@ -230,7 +231,7 @@ class _Base:
                 self.proj.status = "OK"
                 self._update_tasks(self._get_playbook_path(result[0]))
                 self._set_project_modules()
-                self._handle_yaml(self._load_yaml() or dict())
+                self._handle_yaml(self._load_yaml() or {})
                 self._update_slave_inventories(self.proj.slave_inventory.all())
                 self.proj.save()
         except Exception as err:
