@@ -40,8 +40,8 @@ class Git(_VCS):
     }
 
     def __init__(self, *args, **kwargs):
-        super(Git, self).__init__(*args, **kwargs)
-        self.env = self.options.get("GIT_ENV", dict())
+        super().__init__(*args, **kwargs)
+        self.env = self.options.get("GIT_ENV", {})
         self._fetch_map = {
             1 << x: self._fetch_statuses[x] for x in range(8)
         }
@@ -56,14 +56,13 @@ class Git(_VCS):
             self._update_submodules(repo)
         return repo
 
-    @raise_context()
     def _fetch_from_remote(self, repo: git.Repo, env: ENV_VARS_TYPE):
         with repo.git.custom_environment(**env):
-            kwargs = self.options.get("FETCH_KWARGS", dict())
+            kwargs = self.options.get("FETCH_KWARGS", {})
             fetch_method_name = 'fetch'
             if not repo.head.is_detached:
                 fetch_method_name = 'pull'
-            result = getattr(repo.remotes[0], fetch_method_name)(**kwargs)
+            result = getattr(repo.remote('origin'), fetch_method_name)(**kwargs)
             self._update_submodules(repo)
             return result
 
@@ -99,8 +98,8 @@ class Git(_VCS):
             reponame = repo.active_branch.name
         return reponame
 
-    def make_clone(self, env: ENV_VARS_TYPE) -> Tuple[git.Repo, None]:
-        kw = dict(**self.options.get("CLONE_KWARGS", dict()))
+    def make_clone(self, env: ENV_VARS_TYPE) -> Tuple[git.Repo, None]:  # pylint: disable=arguments-renamed
+        kw = dict(**self.options.get("CLONE_KWARGS", {}))
         if self.target_branch:
             kw['branch'] = self.target_branch.replace('tags/', '')
         repo = self.vsc_clone(self.proj.repository, self.path, env=env, **kw)
@@ -111,30 +110,34 @@ class Git(_VCS):
         return repo, None
 
     def _get_or_create_repo(self, env: ENV_VARS_TYPE) -> git.Repo:
-        try:
-            repo = self.get_repo()
-            branch = self.target_branch
-            with raise_context():
-                repo.git.checkout(branch)
-            is_not_detached = not repo.head.is_detached
-            if branch and is_not_detached and repo.active_branch.name != branch:
-                self.delete()
+        repo = self.get_repo()
+        origin = repo.remote('origin')
+        branch = self.target_branch
+
+        if origin.url != self.proj.repository:
+            origin.set_url(self.proj.repository)
+            if repo.head.is_detached:
                 raise git.NoSuchPathError
-        except git.NoSuchPathError:
-            repo = self.make_clone(env)[0]
+
+        if branch not in origin.refs:
+            with repo.git.custom_environment(**env):
+                origin.fetch(**self.options.get('FETCH_KWARGS', {}))
+
+        repo.git.checkout(branch)
         return repo
 
-    def make_update(self, env: ENV_VARS_TYPE) -> Tuple[git.Repo, Any]:
+    def make_update(self, env: ENV_VARS_TYPE) -> Tuple[git.Repo, Any]:  # pylint: disable=arguments-renamed
         try:
             repo = self._get_or_create_repo(env)
+            result = self.vcs_update(repo, env)
         except git.InvalidGitRepositoryError:
             logger.info('Convert project [{}] to GIT.'.format(self.proj.id))
             repo = git.Repo.init(self.path)
             repo.create_remote('origin', self.proj.repository)
             with repo.git.custom_environment(**env):
-                kwargs = self.options.get("FETCH_KWARGS", dict())
+                kwargs = self.options.get("FETCH_KWARGS", {})
                 origin = repo.remote('origin')
-                logger.debug('Fetch remote brances for project [{}].'.format(self.proj.id))
+                logger.debug('Fetch remote branches for project [{}].'.format(self.proj.id))
                 origin.fetch(**kwargs)
                 if not list(origin.refs):
                     config_writer = repo.config_writer()
@@ -148,10 +151,12 @@ class Git(_VCS):
                     logger.debug('Push project [{}] as master.'.format(self.proj.id))
                     repo.git.push('--set-upstream', 'origin', 'master')
 
-        results = repo, self.vcs_update(repo, env)
-        with raise_context():
-            repo.git.checkout(self.target_branch)
-        return results
+            result = self.vcs_update(repo, env)
+        except (git.GitCommandError, git.NoSuchPathError):
+            self.delete()
+            return self.make_clone(env)
+
+        return (repo, result)
 
     def get_revision(self, *args, **kwargs):
         # pylint: disable=unused-argument
@@ -161,7 +166,7 @@ class Git(_VCS):
         return repo.head.object.hexsha
 
     def _with_password(self, tmp, env_vars: ENV_VARS_TYPE) -> ENV_VARS_TYPE:
-        env_vars.update(self.env.get("PASSWORD", dict()))
+        env_vars.update(self.env.get("PASSWORD", {}))
         tmp.write("echo '{}'".format(self.proj.vars["repo_password"]))
         os.chmod(tmp.name, 0o700)
         env_vars["GIT_ASKPASS"] = env_vars.get("GIT_ASKPASS", tmp.name)
@@ -169,7 +174,7 @@ class Git(_VCS):
         return env_vars
 
     def _with_key(self, tmp, env_vars: Dict) -> ENV_VARS_TYPE:
-        env_vars.update(self.env.get("KEY", dict()))
+        env_vars.update(self.env.get("KEY", {}))
         tmp.write(self.proj.vars["repo_key"])
         tmp.close()
         ssh = "ssh -vT -i {} -F /dev/null".format(tmp.name)
@@ -179,13 +184,13 @@ class Git(_VCS):
         return env_vars
 
     def _operate(self, operation, **env_vars):
-        env_vars.update(self.env.get("GLOBAL", dict()))
+        env_vars.update(self.env.get("GLOBAL", {}))
         with tmp_file_context(delete=False) as tmp:
             if self.proj.vars.get("repo_password", None) is not None:
                 env_vars = self._with_password(tmp, env_vars)
             elif self.proj.vars.get("repo_key", None) is not None:
                 env_vars = self._with_key(tmp, env_vars)
-            return super(Git, self)._operate(operation, **env_vars)
+            return super()._operate(operation, **env_vars)
 
     def search_files(self, repo: git.Repo = None, pattern: Text = '**/*') -> Iterable[pathlib.Path]:
         recursive = pattern.startswith('**/')
@@ -213,7 +218,7 @@ class Git(_VCS):
     def get(self) -> Dict[str, str]:
         return {
             res.ref.remote_head: self._fetch_map[res.flags]
-            for res in super(Git, self).get()[1]
+            for res in super().get()[1]
         }
 
     def revision(self) -> Text:
