@@ -7,7 +7,7 @@ POLEMARCH_VERSION = PROJECT_VERSION
 APACHE = False if ("runserver" in sys.argv) else True
 
 # Directory for git projects
-PROJECTS_DIR = main.get("projects_dir", fallback="{LIB}/projects")
+PROJECTS_DIR = main.get("projects_dir", fallback=os.getenv("POLEMARCH_PROJECTS_DIR", "{LIB}/projects"))
 os.makedirs(PROJECTS_DIR) if not os.path.exists(PROJECTS_DIR) else None
 
 # Polemarch apps
@@ -32,79 +32,48 @@ AUTH_PASSWORD_VALIDATORS += [
 ]
 
 # API settings
-VST_API_VERSION = 'v2'
+VST_API_VERSION = 'v3'
 
 REST_FRAMEWORK["DEFAULT_PERMISSION_CLASSES"] = [
-    "{}.api.{}.permissions.ModelPermission".format(VST_PROJECT_LIB_NAME, VST_API_VERSION),
+    "{}.api.v2.permissions.ModelPermission".format(VST_PROJECT_LIB_NAME),
 ]
 
 API_URL = VST_API_URL
 DEFAULT_API_URL = "/{}/{}".format(API_URL, VST_API_VERSION)
 API = {
-    VST_API_VERSION: OrderedDict(
+    'v2': OrderedDict(
         project={'view': '{}.api.v2.views.ProjectViewSet'.format(VST_PROJECT_LIB_NAME)},
         community_template={'view': '{}.api.v2.views.ProjectTemplateViewSet'.format(VST_PROJECT_LIB_NAME)},
         inventory={'view': '{}.api.v2.views.InventoryViewSet'.format(VST_PROJECT_LIB_NAME)},
         group={'view': '{}.api.v2.views.GroupViewSet'.format(VST_PROJECT_LIB_NAME)},
         host={'view': '{}.api.v2.views.HostViewSet'.format(VST_PROJECT_LIB_NAME)},
         history={'view': '{}.api.v2.views.HistoryViewSet'.format(VST_PROJECT_LIB_NAME), "op_types": ['get', 'del', 'mod']},
-        _bulk={'view': '{}.api.v2.views.BulkViewSet'.format(VST_PROJECT_LIB_NAME), 'type': 'view'},
         user={'view': '{}.api.v2.views.UserViewSet'.format(VST_PROJECT_LIB_NAME)},
         team={'view': '{}.api.v2.views.TeamViewSet'.format(VST_PROJECT_LIB_NAME)},
         token={'view': '{}.api.v2.views.TokenView'.format(VST_PROJECT_LIB_NAME), 'type': 'view'},
         hook={'view': '{}.api.v2.views.HookViewSet'.format(VST_PROJECT_LIB_NAME)},
-        stats={'view': '{}.api.v2.views.StatisticViewSet'.format(VST_PROJECT_LIB_NAME), 'op_types': ['get']})
+        stats={'view': '{}.api.v2.views.StatisticViewSet'.format(VST_PROJECT_LIB_NAME), 'op_types': ['get']}
+    )
+}
+API[VST_API_VERSION] = {
+    **API['v2'],
+    'group': {
+        'view': '{}.api.v3.views.GroupViewSet'.format(VST_PROJECT_LIB_NAME)
+    },
+    'inventory': {
+        'view': '{}.api.v3.views.InventoryViewSet'.format(VST_PROJECT_LIB_NAME)
+    },
+    'project': {
+        'view': '{}.api.v3.views.ProjectViewSet'.format(VST_PROJECT_LIB_NAME)
+    },
 }
 
-PROJECT_GUI_MENU = [
-    {
-        'name': 'Projects',
-        'url': '/project',
-        'span_class': 'fas fa-cubes',
-    },
-    {
-        'name': 'Community',
-        'url': '/community_template',
-        'span_class': 'fa fa-cloud',
-    },
-    {
-        'name': 'Inventories',
-        'url': '/inventory',
-        'span_class': 'fa fa-folder',
-        'sublinks': [
-            {
-                'name': 'Groups',
-                'url': '/group',
-                'span_class': 'fas fa-server',
-            },
-            {
-                'name': 'Hosts',
-                'url': '/host',
-                'span_class': 'fas fa-hdd',
-            },
-        ]
-    },
-    {
-        'name': 'History',
-        'url': '/history',
-        'span_class': 'fa fa-calendar',
-    },
-    {
-        'name': 'System',
-        'span_class': 'fa fa-cog',
-        'sublinks': [
-            {
-                'name': 'Users',
-                'url': '/user',
-                'span_class': 'fa fa-user',
-            },
-            {
-                'name': 'Hooks',
-                'url': '/hook',
-                'span_class': 'fa fa-plug'
-            },
-        ]
-    },
+PROJECT_GUI_MENU = []
+
+OPENAPI_HOOKS = [
+    'polemarch.main.openapi.set_gui_menu_ce',
+    'polemarch.main.openapi.set_inventory_field',
+    'polemarch.main.openapi.set_periodic_task_variable_value_field',
 ]
 
 SWAGGER_SETTINGS['DEFAULT_INFO'] = '{}.api.v2.swagger.api_info'.format(VST_PROJECT_LIB_NAME)
@@ -245,7 +214,7 @@ TASKS_HANDLERS = {
     "REPO": {
         "BACKEND": "{}.main.tasks.tasks.RepoTask".format(VST_PROJECT_LIB_NAME)
     },
-    "SCHEDUER": {
+    "SCHEDULER": {
         "BACKEND": "{}.main.tasks.tasks.ScheduledTask".format(VST_PROJECT_LIB_NAME)
     },
     "MODULE": {
@@ -256,6 +225,11 @@ TASKS_HANDLERS = {
     },
 }
 
+NOTIFY_WITHOUT_QUEUE_MODELS = [
+    'main.History',
+    'main.Project',
+]
+
 CLONE_RETRY = rpc.getint('clone_retry_count', fallback=5)
 
 # ACL settings
