@@ -13,8 +13,8 @@ class URLValidator(OldURLValidator):
 
 
 path_validator = RegexValidator(
-    regex=r'(^[^\.]+?$)|(^([\.][\\//])?([\.]*[\w\d\-_]+?[\.]*[\w\d\-_]*[\\//]*)+?$)',
-    message='Incompatible symbols `..`.'
+    regex=r'^([\w\d_\-]+|[\.][\/\w\d_\-\+=\^,])(\.?[\w\d\-\/\\_()\+=\^,]+)+$',
+    message=_('Invalid path. Path must not contain ".." or any special characters.')
 )
 
 
